searchData={"items":[{"type":"module","title":"ExDoctor","doc":"This module defines records from the `:tr` module, allowing to use them in `iex`.","ref":"ExDoctor.html"},{"type":"macro","title":"ExDoctor.msg/1","doc":"","ref":"ExDoctor.html#msg/1"},{"type":"macro","title":"ExDoctor.msg/2","doc":"","ref":"ExDoctor.html#msg/2"},{"type":"macro","title":"ExDoctor.node/1","doc":"","ref":"ExDoctor.html#node/1"},{"type":"macro","title":"ExDoctor.node/2","doc":"","ref":"ExDoctor.html#node/2"},{"type":"macro","title":"ExDoctor.tr/1","doc":"","ref":"ExDoctor.html#tr/1"},{"type":"macro","title":"ExDoctor.tr/2","doc":"","ref":"ExDoctor.html#tr/2"},{"type":"module","title":"ExDoctor.Example","doc":"","ref":"ExDoctor.Example.html"},{"type":"function","title":"ExDoctor.Example.fib/1","doc":"","ref":"ExDoctor.Example.html#fib/1"},{"type":"function","title":"ExDoctor.Example.sleepy_factorial/1","doc":"","ref":"ExDoctor.Example.html#sleepy_factorial/1"},{"type":"extras","title":"ExDoctor","doc":"# ExDoctor\n\nLightweight tracing, debugging and profiling tool, that collects traces from your system in an ETS table, putting minimal impact on the system.\nAfter collecting the traces, you can query and analyse them.\nBy separating data collection from analysis, this tool helps you limit unnecessary repetition and guesswork.","ref":"readme.html"},{"type":"extras","title":"Quick start - ExDoctor","doc":"To quickly try it out right now, copy & paste the following to your `iex`:\n\n```elixir\n:ssl.start; :inets.start; for p <- [\"erlang_doctor/master/src/tr.erl\", \"ex_doctor/main/lib/ex_doctor.ex\"] do {:ok, {{_, 200, _}, _, src}} = :httpc.request(\"https://raw.githubusercontent.com/chrzaszcz/\" <> p); tp = \"/tmp/\" <> Path.basename(p); File.write!(tp, src); c tp end; import ExDoctor; :tr.start\n```\n\nThis snippet downloads, compiles and starts two modules:\n\n- `:tr` is the main module of [Erlang Doctor](https://github.com/chrzaszcz/erlang_doctor), which provides all the functionality.\n- `ExDoctor` is a small Elixir module, which allows using the Erlang records defined in `tr.hrl`.\n\nThe Erlang records are used to allow quick and easy pattern-matching, which is used very frequently in `erlang_doctor`.\nMaps are not used, because they can be a lot slower and consume more memory (this is verified by benchmarks).\n\nThe easiest way to use it is the following:\n\n```elixir\n:tr.trace([YourModule])\nYourModule.some_function()\n:tr.select\n```\n\nYou should see the collected traces for the call and return of `YourModule.some_function/0`.\n\n#","ref":"readme.html#quick-start"},{"type":"extras","title":"Include it as a dependency - ExDoctor","doc":"The [package](https://hex.pm/packages/ex_doctor) can be installed by adding `ex_doctor` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_doctor, \"~> 0.2.3\"}\n  ]\nend\n```\n\n#","ref":"readme.html#include-it-as-a-dependency"},{"type":"extras","title":"Use it during development - ExDoctor","doc":"You can make Erlang Doctor available in the Erlang/Rebar3 shell during development by cloning it to `EX_DOCTOR_PATH`,\ncompiling it with `mix`, and loading it in your `~/.iex.exs` file:\n\n```elixir\nCode.append_path(\"EX_DOCTOR_PATH/_build/dev/lib/erlang_doctor/ebin\")\nCode.append_path(\"EX_DOCTOR_PATH/_build/dev/lib/ex_doctor/ebin\")\nimport ExDoctor\nCode.ensure_loaded!(:tr)\n```","ref":"readme.html#use-it-during-development"},{"type":"extras","title":"Tracing: data collection - ExDoctor","doc":"You can follow the examples on your own - just call `iex -S mix` in `EX_DOCTOR_PATH`,\nand execute the numbered commands in the same order.\n\n#","ref":"readme.html#tracing-data-collection"},{"type":"extras","title":"Setting up: `start`, `start_link` - ExDoctor","doc":"In our case ExDoctor is automatically started by `mix`, but if you need to start it yourself, call `:tr.start()`.\nThere is `:tr.start_link()` as well, but it is intended for use with the whole application.\nBoth functions can also take an argument, which is a map of options with the following keys:\n\n- `tab`: collected traces are stored in an ETS table with this name (default: `:trace`),\n- `limit`: maximum number of traces in the table - when it is reached, tracing is stopped (default: no limit).\n\nLet's set up an alias for the `Example` module, because it will be used very often:\n\n```elixir\niex(1)> alias ExDoctor.Example\nExDoctor.Example\n```\n\n#","ref":"readme.html#setting-up-start-start_link"},{"type":"extras","title":"Tracing with `trace` - ExDoctor","doc":"To trace function calls for given modules, use `:tr.trace/1`, providing a list of traced modules:\n\n```elixir\niex(2)> :tr.trace([Example])\n:ok\n```\n\nYou can provide `{module, function, arity}` tuples in the list as well.\nTo get a list of all modules from an application, use `:tr.app_modules/1`.\n`:tr.trace(:tr.app_modules(:your_app))` would trace all modules from `your_app`.\nThere is a shortcut as well: `:tr.trace_app(:your_app)`.\nIf you want to trace selected processes instead of all of them, you can use\n`:tr.trace(modules, pids)`, which is a shortcut for `:tr.trace(%{modules: modules, pids: pids})`.\nIn fact, `:tr.trace(modules)` is a shortcut for `:tr.trace(%{modules: modules})`,\nand the `trace/1` function accepts a map of options with the following keys:\n\n- `modules`: a list of module names or `{module, function, arity}` tuples. The list is empty by default.\n- `pids`: a list of Pids of processes to trace, or `:all` (default) to trace all processes.\n- `msg`: `:none` (default), `:all`, `:send` or `:recv`. Specifies which message events will be traced. By default no messages are traced.\n- `msg_trigger`: `:after_traced_call` (default) or `:always`. By default, traced messages in each process are stored after the first traced function call in that process. The goal is to limit the number of traced messages, which can be huge in the entire Erlang system. If you want all messages, set it to `:always`.\n\nNow we can call some functions - let's trace the following function call.\nIt calculates the factorial recursively and sleeps 1 ms between each step.\n\n```elixir\niex(3)> Example.sleepy_factorial(3)\n6\n```\n\n#","ref":"readme.html#tracing-with-trace"},{"type":"extras","title":"Stopping tracing - ExDoctor","doc":"You can stop tracing with the following function:\n\n```elixir\niex(4)> :tr.stop_tracing()\n:ok\n```\n\nIt's good to stop it as soon as possible to avoid accumulating too many traces in the ETS table.\nUsage of `tr` on production systems is risky, but if you have to do it, start and stop the tracer in the same command,\ne.g. for one second with:\n\n```elixir\n:tr.trace(modules); :timer.sleep(1000); :tr.stop_tracing()\n```","ref":"readme.html#stopping-tracing"},{"type":"extras","title":"Debugging: data analysis - ExDoctor","doc":"The collected traces are stored in an ETS table (default name: `:trace`).\nThey are stored as `tr` records with the following fields:\n\n- `index`: trace identifier, auto-incremented for each received trace.\n- `pid`: process ID associated with the trace.\n- `event`: `:call`, `:return` or `:exception` for function traces; `:send` or `:recv` for messages.\n- `mfa`: `{module, function, arity}` for function traces; `:no_mfa` for messages.\n- `data`: argument list (for calls), returned value (for returns) or class and value (for exceptions).\n- `timestamp` in microseconds.\n- `info`: For function traces and `:recv` events it is `:no_info`. For `:send` events it is a `{to, exists}` tuple, where `to` is the recipient pid, and `exists` is a boolean indicating if the recipient process existed.\n\nYou can load the record definitions with `import ExDoctor`, but in our case `mix` has done it for us.\nThe snippets shown at the top of this README do it as well.\n\n#","ref":"readme.html#debugging-data-analysis"},{"type":"extras","title":"Trace selection: `select` - ExDoctor","doc":"Use `:tr.select/0` to select all collected traces, which include a system call to `__info__/1`\nfollowed by the call to `sleepy_factorial/1`.\n\n```elixir\niex(5)> :tr.select()\n[\n  {:tr, 1, #PID<0.187.0>, :call, {ExDoctor.Example, :__info__, 1},\n   [:deprecated], 1705413018330494, :no_info},\n  {:tr, 2, #PID<0.187.0>, :return, {ExDoctor.Example, :__info__, 1}, [],\n   1705413018330501, :no_info},\n  {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [3],\n   1705413018330532, :no_info},\n  {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [2],\n   1705413018332522, :no_info},\n  {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [1],\n   1705413018334514, :no_info},\n  {:tr, 6, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [0],\n   1705413018336506, :no_info},\n  {:tr, 7, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 1,\n   1705413018338509, :no_info},\n  {:tr, 8, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 1,\n   1705413018338511, :no_info},\n  {:tr, 9, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 2,\n   1705413018338512, :no_info},\n  {:tr, 10, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 6,\n   1705413018338513, :no_info}\n]\n```\n\nThe `:tr.select/1` function accepts a fun that is passed to `:ets.fun2ms/1`.\nThis way you can limit the selection to specific items and select only some fields from the `tr` record:\n\n```elixir\niex(6)> :tr.select(fn tr(event: :call, data: [n]) when is_integer(n) -> n end)\n[3, 2, 1, 0]\n```\n\nUse `:tr.select/2` to further filter the results by searching for a term in the `data` field\n(recursively searching in lists, tuples and maps).\n\n```elixir\niex(7)> :tr.select(fn t -> t end, 2)\n[\n  {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [2],\n   1705413018332522, :no_info},\n  {:tr, 9, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 2,\n   1705413018338512, :no_info}\n]\n```\n\n#","ref":"readme.html#trace-selection-select"},{"type":"extras","title":"Trace filtering: `filter` - ExDoctor","doc":"Sometimes it might be easier to use `:tr.filter/1`, because it can accept any function as the argument.\nYou can use `:tr.contains_data/2` to search for a term like in the example above.\n\n```elixir\niex(8)> traces = :tr.filter(fn t -> :tr.contains_data(2, t) end)\n[\n  {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [2],\n   1705413018332522, :no_info},\n  {:tr, 9, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1}, 2,\n   1705413018338512, :no_info}\n]\n```\n\nThe provided function is a predicate, which has to return `:true` for the matching traces.\nFor other traces it can return another value, or even raise an exception:\n\n```elixir\niex(9)> :tr.filter(fn tr(data: [2]) -> :true end)\n[\n  {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [2],\n   1705413018332522, :no_info}\n]\n```\n\nThere is also `:tr.filter/2`, which can be used to search in a different table than the current one - or in a list:\n\n```elixir\niex(10)> :tr.filter(fn tr(event: :call) -> :true end, traces)\n[\n  {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [2],\n   1705413018332522, :no_info}\n]\n```\n\n#","ref":"readme.html#trace-filtering-filter"},{"type":"extras","title":"Tracebacks for filtered traces: `tracebacks` - ExDoctor","doc":"To find the tracebacks (stack traces) for matching traces, use `:tr.tracebacks/1`:\n\n```elixir\niex(11)> :tr.tracebacks(fn tr(data: 1) -> true end)\n[\n  [\n    {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [1], 1705413018334514, :no_info},\n    {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [2], 1705413018332522, :no_info},\n    {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [3], 1705413018330532, :no_info}\n  ]\n]\n```\n\nNote, that by specifying `data: 1` we are only matching return traces, as call traces always have a list in `data`.\nOnly one traceback is returned. It starts with a call that returned `1`. What follows is the stack trace for this call.\n\nOne can notice that the call for 0 also returned 1, but the call tree got pruned - whenever two tracebacks overlap, only the shorter one is left.\nYou can change this by returning tracebacks for all matching traces even if they overlap, setting the `output` option to `:all`. All options are specified in the second argument, which is a map:\n\n```elixir\niex(12)> :tr.tracebacks(fn tr(data: 1) -> true end, %{output: :all})\n[\n  [\n    {:tr, 6, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [0], 1705413018336506, :no_info},\n    {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [1], 1705413018334514, :no_info},\n    {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [2], 1705413018332522, :no_info},\n    {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [3], 1705413018330532, :no_info}\n  ],\n  [\n    {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [1], 1705413018334514, :no_info},\n    {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [2], 1705413018332522, :no_info},\n    {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [3], 1705413018330532, :no_info}\n  ]\n]\n```\n\nThe third possibility is `:longest`, which does the opposite of pruning, leaving only the longest tracabacks when they overlap:\n\n```elixir\niex(13)> :tr.tracebacks(fn tr(data: 1) -> true end, %{output: :longest})\n[\n  [\n    {:tr, 6, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [0], 1705413018336506, :no_info},\n    {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [1], 1705413018334514, :no_info},\n    {:tr, 4, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [2], 1705413018332522, :no_info},\n    {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [3], 1705413018330532, :no_info}\n  ]\n]\n```\n\nAll possible options for `tracebacks/2`:\n\n- `tab` is the table or list which is like the second argument of `:tr.filter/2`,\n- `output` - `:shortest` (default), `:all`, `:longest` - see above.\n- `format` - `:list` (default), `:tree` - returns a call tree instead of a list of tracebacks. Trees don't distinguish between `all` and `longest` output formats.\n- `order` - `:top_down` (default), `:bottom_up` - call order in each tracaback; only for the `:list` format.\n- `limit` - positive integer or `:infinity` (default) - limits the number of matched traces. The actual number of tracebacks returned can be smaller unless `output` is set ot `:all`\n\nThere are also functions `traceback/1` and `traceback/2`. They set `limit` to one and return only one trace if it exists. The options for `traceback/2` are the same as for `traceback/2` except `limit` and `format`. Additionally, it is possible to pass a `tr` record (or an index) directly to `traceback/1` to obtain the traceback for the provided trace event.\n\n#","ref":"readme.html#tracebacks-for-filtered-traces-tracebacks"},{"type":"extras","title":"Trace ranges for filtered traces: `ranges` - ExDoctor","doc":"To get a list of traces between each matching call and the corresponding return, use `:tr.ranges/1`:\n\n```elixir\niex(14)> :tr.ranges(fn tr(data: [1]) -> true end)\n[\n  [\n    {:tr, 5, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [1], 1705413018334514, :no_info},\n    {:tr, 6, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1},\n     [0], 1705413018336506, :no_info},\n    {:tr, 7, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1},\n     1, 1705413018338509, :no_info},\n    {:tr, 8, #PID<0.187.0>, :return, {ExDoctor.Example, :sleepy_factorial, 1},\n     1, 1705413018338511, :no_info}\n  ]\n]\n```\n\nThere is also `:tr.ranges/2` - it accepts a map of options with the following keys:\n\n- `tab` is the table or list which is like the second argument of `:tr.filter/2`,\n- `max_depth` is the maximum depth of nested calls. A message event also adds 1 to the depth.\n    You can use `%{max_depth: 1}` to see only the top-level call and the corresponding return.\n\nThere are two additional functions: `:tr.range/1` and `:tr.range/2`, which return only one range if it exists. It is possible to pass a `tr` record or an index to `:tr.range/1` as well.\n\n#","ref":"readme.html#trace-ranges-for-filtered-traces-ranges"},{"type":"extras","title":"Calling function from a trace: `do` - ExDoctor","doc":"It is easy to replay a particular function call with `:tr.do/1`:\n\n```elixir\niex(15)> [t] = :tr.filter(fn tr(data: [3]) -> true end)\n[\n  {:tr, 3, #PID<0.187.0>, :call, {ExDoctor.Example, :sleepy_factorial, 1}, [3],\n   1705413018330532, :no_info}\n]\niex(16)> :tr.do(t)\n6\n```\n\nThis is useful e.g. for checking if a bug has been fixed without running the whole test suite.\nThis function can be called with an index as the argument.\n\n#","ref":"readme.html#calling-function-from-a-trace-do"},{"type":"extras","title":"Getting a single trace for the index: `lookup` - ExDoctor","doc":"Use `:tr.lookup/1` to obtain the trace for an index.","ref":"readme.html#getting-a-single-trace-for-the-index-lookup"},{"type":"extras","title":"Profiling - ExDoctor","doc":"You can quickly get a hint about possible bottlenecks and redundancies in your system with function call statistics.\n\n#","ref":"readme.html#profiling"},{"type":"extras","title":"Call statistics: `call_stat` - ExDoctor","doc":"The argument of `:tr.call_stat/1` is a function that returns a key by which the traces are grouped.\nThe simplest way to use this function is to look at the total number of calls and their time.\nTo do this, we group all calls under one key, e.g. `total`:\n\n```elixir\niex(17)> :tr.call_stat(fn _ -> :total end)\n%{total: {5, 7988, 7988}}\n```\n\nValues of the returned map have the following format (time is in microseconds):\n\n```{call_count, acc_time, own_time}```\n\nIn the example there are four calls, which took 7981 microseconds in total.\nFor nested calls we only take into account the outermost call, so this means that the whole calculation took 7.981 ms.\nLet's see how this looks like for individual steps - we can group the stats by the function argument:\n\n```elixir\niex(18)> :tr.call_stat(fn tr(data: [n]) -> n end)\n%{\n  0 => {1, 2003, 2003},\n  1 => {1, 3997, 1994},\n  2 => {1, 5990, 1993},\n  3 => {1, 7981, 1991},\n  :deprecated => {1, 7, 7}\n}\n```\n\nYou can use the provided function to do filtering as well - let's make the output cleaner\nby filtering out the unwanted call to `__info__(:deprecated)`:\n\n```elixir\niex(19)> :tr.call_stat(fn tr(data: [n]) when is_integer(n) -> n end)\n%{\n  0 => {1, 2003, 2003},\n  1 => {1, 3997, 1994},\n  2 => {1, 5990, 1993},\n  3 => {1, 7981, 1991}\n}\n```\n\n#","ref":"readme.html#call-statistics-call_stat"},{"type":"extras","title":"Sorted call statistics: `sorted_call_stat` - ExDoctor","doc":"You can sort the call stat by accumulated time, descending:\n\n```elixir\niex(20)> :tr.sorted_call_stat(fn tr(data: [n]) when is_integer(n) -> n end)\n[{3, 1, 7981, 1991}, {2, 1, 5990, 1993}, {1, 1, 3997, 1994}, {0, 1, 2003, 2003}]\n```\n\nThe first element of each tuple is the key, the rest is the same as above.\nTo pretty-print it, use `:tr.print_sorted_call_stat/2`.\nThe second argument limits the table row number, e.g. we can only print the top 3 items:\n\n```elixir\niex(21)> :tr.print_sorted_call_stat(fn tr(data: [n]) when is_integer(n) -> n end, 3)\n3  1  7981  1991\n2  1  5990  1993\n1  1  3997  1994\n:ok\n```\n\n#","ref":"readme.html#sorted-call-statistics-sorted_call_stat"},{"type":"extras","title":"Call tree statistics: `top_call_trees` - ExDoctor","doc":"This function makes it possible to detect complete call trees that repeat several times,\nwhere corresponding function calls and returns have the same arguments and return values, respectively.\nWhen such functions take a lot of time and do not have useful side effects, they can be often optimized.\n\nAs an example, let's trace the call to a function which calculates the 4th element of the Fibonacci Sequence\nin a recursive way. The `trace` table should be empty, so let's clean it up first:\n\n```elixir\niex(22)> :tr.clean()\n:ok\niex(23)> :tr.trace([{Example, :fib, 1}])\nok\niex(24)> Example.fib(4)\n3\niex(25)> :tr.stop_tracing()\n:ok\n```\n\nNow it is possible to print the most time consuming call trees that repeat at least twice:\n\n```elixir\niex(26)> :tr.top_call_trees()\n[\n  {13, 2,\n   {:node, ExDoctor.Example, :fib, [2],\n    [\n      {:node, ExDoctor.Example, :fib, [1], [], {:return, 1}},\n      {:node, ExDoctor.Example, :fib, [0], [], {:return, 0}}\n    ], {:return, 1}}},\n  {5, 3, {:node, ExDoctor.Example, :fib, [1], [], {:return, 1}}}\n]\n```\n\nThe resulting list contains tuples `{time, count, tree}` where `time` is the accumulated time (in microseconds) spent in the tree,\nand `count` is the number of times the tree repeated. The list is sorted by `time`, descending.\nIn the example above `fib(2)` was called twice and `fib(1)` was called 3 times,\nwhat already shows that the recursive implementation is suboptimal.\n\nThere is also `top_call_trees/1` that takes a map of options with the following keys:\n- `output` is `:reduced` by default, but it can be set to `:complete` where subtrees of already listed trees are also listed.\n- `min_count` is the minimum number of times a tree has to occur to be listed, the default is 2.\n- `min_time` is the minimum accumulated time for a tree, by default there is no minimum.\n- `max_size` is the maximum number of trees presented, the default is 10.\n\nAs an exercise, try calling `:tr.top_call_trees(%{min_count: 1000})` for `fib(20)`.","ref":"readme.html#call-tree-statistics-top_call_trees"},{"type":"extras","title":"Exporting and importing traces - ExDoctor","doc":"To get the current table name, use `:tr.tab/0`:\n\n```elixir\niex(27)> :tr.tab()\n:trace\n```\n\nTo switch to a new table, use `:tr.set_tab/1`. The table need not exist.\n\n```elixir\niex(28)> :tr.set_tab(:tmp)\n:ok\n```\n\nNow you can collect traces to the new table without changing the original one.\n\n```elixir\niex(29)> :tr.trace([Enum]); Enum.to_list(1..10); :tr.stop_tracing()\n:ok\niex(30)> :tr.select()\n[\n  (...)\n]\n```\n\nYou can dump the current table to file:\n\n```elixir\niex(31)> :tr.dump(\"tmp.ets\")\n:ok\n```\n\nIn a new `iex` session we can load the data with `:tr.load/1`. This will set the current table name to `:tmp`.\n\n```elixir\niex(1)> :tr.load(\"tmp.ets\")\n{:ok, :tmp}\niex(2)> :tr.select()\n[\n  (...)\n]\niex(3)> :tr.tab()\n:tmp\n```\n\nFinally, you can remove all traces from the ETS table with `:tr.clean/0`.\n\n```elixir\niex(4)> :tr.clean()\n:ok\n```\n\nTo stop `ExDoctor`, just call `:tr.stop/0`.","ref":"readme.html#exporting-and-importing-traces"}],"content_type":"text/markdown"}